@@ -1,13 +1,14 @@
 from __future__ import absolute_import
-from lxml import etree
-from lxml.etree import Element
-from lxml.builder import E
-from datetime import datetime, timedelta
+
 import base64
-from binascii import Error as BinasciiError
-from dateutil import parser, tz
-import uuid
 import struct
+import uuid
+from binascii import Error as BinasciiError
+from datetime import datetime, timedelta
+
+from dateutil import parser, tz
+from lxml import etree
+from lxml.builder import E
 
 
 class BaseElement(object):
@@ -22,13 +23,11 @@ class BaseElement(object):
         )
         if icon:
             self._element.append(E.IconID(icon))
-        current_time_str = self._encode_time(datetime.utcnow())
+        current_time_str = self._encode_time(datetime.now())
         if expiry_time:
-            expiry_time_str = self._encode_time(
-                self._datetime_to_utc(expiry_time)
-            )
+            expiry_time_str = self._encode_time(expiry_time)
         else:
-            expiry_time_str = self._encode_time(datetime.utcnow())
+            expiry_time_str = current_time_str
 
         self._element.append(
             E.Times(
@@ -71,11 +70,15 @@ class BaseElement(object):
 
     @property
     def uuid(self):
-        return self._get_subelement_text('UUID')
+        """Returns uuid of this element as a uuid.UUID object"""
+        b64_uuid = self._get_subelement_text('UUID')
+        return uuid.UUID(bytes=base64.b64decode(b64_uuid))
 
     @uuid.setter
-    def uuid(self, value):
-        return self._set_subelement_text('UUID', value)
+    def uuid(self, uuid):
+        """Set element uuid. `uuid` is a uuid.UUID object"""
+        b64_uuid = base64.b64encode(uuid.bytes).decode('utf-8')
+        return self._set_subelement_text('UUID', b64_uuid)
 
     @property
     def icon(self):
@@ -126,21 +129,20 @@ class BaseElement(object):
                 return (
                     datetime(year=1, month=1, day=1, tzinfo=tz.gettz('UTC')) +
                     timedelta(
-                        seconds = struct.unpack('<Q', base64.b64decode(text))[0]
+                        seconds=struct.unpack('<Q', base64.b64decode(text))[0]
                     )
                 )
             except BinasciiError:
                 return parser.parse(
                     text,
-                    tzinfos={'UTC':tz.gettz('UTC')}
+                    tzinfos={'UTC': tz.gettz('UTC')}
                 )
         else:
             return parser.parse(
                 text,
-                tzinfos={'UTC':tz.gettz('UTC')}
+                tzinfos={'UTC': tz.gettz('UTC')}
             )
 
-
     def _get_times_property(self, prop):
         times = self._element.find('Times')
         if times is not None:
@@ -173,7 +175,6 @@ class BaseElement(object):
             return self._datetime_to_utc(datetime.utcnow()) > self._datetime_to_utc(self.expiry_time)
         return False
 
-
     @property
     def expiry_time(self):
         return self._get_times_property('ExpiryTime')
