@@ -1,21 +1,18 @@
 # FIXME python2
-from __future__ import unicode_literals
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 from future.utils import python_2_unicode_compatible
 
+import logging
 from copy import deepcopy
+from datetime import datetime
+
+from lxml.builder import E
 from lxml.etree import Element, _Element
 from lxml.objectify import ObjectifiedElement
-from lxml.builder import E
-import logging
-from datetime import datetime
-from collections import namedtuple
-import gzip
-import base64
 
-from pykeepass.baseelement import BaseElement
-import pykeepass.group
 import pykeepass.attachment
+import pykeepass.group
+from pykeepass.baseelement import BaseElement
 
 logger = logging.getLogger(__name__)
 reserved_keys = [
@@ -113,11 +110,14 @@ class Entry(BaseElement):
         )
         self._element.append(element)
 
-        return pykeepass.attachment.Attachment(element=element)
+        return pykeepass.attachment.Attachment(element=element, kp=self._kp)
 
     def delete_attachment(self, attachment):
         attachment.delete()
 
+    def deref(self, attribute):
+        return self._kp.deref(getattr(self, attribute))
+
     @property
     def title(self):
         return self._get_string_field('Title')
@@ -168,7 +168,7 @@ class Entry(BaseElement):
 
     @property
     def tags(self):
-        val =  self._get_subelement_text('Tags')
+        val = self._get_subelement_text('Tags')
         return val.split(';') if val else val
 
     @tags.setter
@@ -181,6 +181,8 @@ class Entry(BaseElement):
     def history(self):
         if self._element.find('History') is not None:
             return [Entry(element=x, kp=self._kp) for x in self._element.find('History').findall('Entry')]
+        else:
+            return []
 
     @history.setter
     def history(self, value):
@@ -216,25 +218,18 @@ class Entry(BaseElement):
             return parent.tag == 'History'
         return False
 
-
     @property
     def path(self):
         # The root group is an orphan
-        if self.is_a_history_entry:
-            pentry = Entry(
-                element=self._element.getparent().getparent(),
-                kp=self._kp
-            ).title
-            return '[History of: {}]'.format(pentry)
         if self.parentgroup is None:
             return None
         p = self.parentgroup
-        ppath = ''
+        path = [self.title]
         while p is not None and not p.is_root_group:
-            if p.name is not None: # dont make the root group appear
-                ppath = '{}/{}'.format(p.name, ppath)
+            if p.name is not None:  # dont make the root group appear
+                path.insert(0, p.name)
             p = p.parentgroup
-        return '{}{}'.format(ppath, self.title)
+        return path
 
     def set_custom_property(self, key, value):
         assert key not in reserved_keys, '{} is a reserved key'.format(key)
@@ -260,13 +255,26 @@ class Entry(BaseElement):
             props[k] = self._get_string_field(k)
         return props
 
+    def ref(self, attribute):
+        """Create reference to an attribute of this element."""
+        attribute_to_field = {
+            'title': 'T',
+            'username': 'U',
+            'password': 'P',
+            'url': 'A',
+            'notes': 'N',
+            'uuid': 'I',
+        }
+        return '{{REF:{}@I:{}}}'.format(attribute_to_field[attribute], self.uuid.hex.upper())
+
     def touch(self, modify=False):
         '''
         Update last access time of an entry
         '''
-        self._element.Times.LastAccessTime = datetime.utcnow()
+        now = datetime.now()
+        self.atime = now
         if modify:
-            self._element.Times.LastModificationTime = datetime.utcnow()
+            self.mtime = now
 
     def save_history(self):
         '''
@@ -283,4 +291,7 @@ class Entry(BaseElement):
             self._element.append(history)
 
     def __str__(self):
-        return 'Entry: "{} ({})"'.format(self.path, self.username)
+        pathstr = '/'.join(self.path)
+        if self.is_a_history_entry:
+            return '[History of: {}]'.format(pathstr)
+        return 'Entry: "{} ({})"'.format(pathstr, self.username)
